/***********************************************************************/
/*                                                                     */
/*   svm_struct_api.h                                                  */
/*                                                                     */
/*   Definition of API for attaching implementing SVM learning of      */
/*   structures (e.g. parsing, multi-label classification, HMM)        */ 
/*                                                                     */
/*   Author: Thorsten Joachims                                         */
/*   Date: 13.10.03                                                    */
/*                                                                     */
/*   Copyright (c) 2003  Thorsten Joachims - All rights reserved       */
/*                                                                     */
/*   This software is available for non-commercial use only. It must   */
/*   not be modified and distributed without prior permission of the   */
/*   author. The author is not responsible for implications from the   */
/*   use of this software.                                             */
/*                                                                     */
/***********************************************************************/

#ifndef svm_struct_api_types
#define svm_struct_api_types

# define INST_NAME          "SVM-Align"
# define INST_VERSION       "V2.00"
# define INST_VERSION_DATE  "15.07.04"

typedef struct pattern {
  int   id;
  int   *original;     /* reference sequence      */
  int   *homolog;      /* similar sequence        */
  int   **decoy;       /* non-similar sequences   */
  int   numDecoys;     /* number of decoys        */
} PATTERN;

typedef struct label {
  int   id;
  int   *align_h;      /* optimal alignment with homolog */
  int   decoy_id;      /* used for finding highest incorrect sequence (mini-hack)*/
} LABEL;

typedef struct structmodel {
  double *w;          /* pointer to the learned weights (in svm_model) */
  MODEL  *svm_model;  /* the learned SVM model */
  long   sizePhi;     /* maximum number of weights in w */

  int    alphabetsize;   /* size of alphabet */
  float  subst_scale;    /* factor by which to scale sub matrix */

} STRUCTMODEL;

typedef struct struct_learn_parm {
  double epsilon;              /* precision for which to solve
				  quadratic program */
  double newconstretrain;      /* number of new constraints to
				  accumulate before recomputing the QP
				  solution */
  double C;                    /* trade-off between margin and loss */
  char   custom_argv[20][300]; /* string set with the -u command line option */
  int    custom_argc;          /* number of -u command line options */
  int    slack_norm;           /* norm to use in objective function
                                  for slack variables; 1 -> L1-norm, 
				  2 -> L2-norm */
  int    loss_type;            /* selected loss function from -r
				  command line option. Select between
				  slack rescaling (1) and margin
				  rescaling (2) */
  int    loss_function;        /* select between different loss
				  functions via -l command line
				  option */
  int    initialize_from_file; /* flag for initialization */
  char   init_model_file[60];  /* name of initialization file */

  /* Parameters specific to alignment implementation */
  int    alphabetsize;            /* size of alphabet */
  float  subst_scale;             /* factor by which to scale sub matrix */
  int    ind_gap_costs;           /* whether or not each character gets its own gap score */
  int    gap_extension;           /* whether or not to use gap extension */
  int    ins_del_score;           /* use separate scores for insert, delete */
  int    fix_subst_matrix;        /* whether or not to learn subst matrix */
  double *w_subst;                /* place to maintain fixed weights, if fix_subst_matrix=1*/
  int    use_all_param;           /* shortcut for using all parameters */

} STRUCT_LEARN_PARM;
#endif
